import { CLIEngine } from "eslint";
import chokidar from "chokidar";
import { extname } from "path";
import { debounce } from "throttle-debounce";
import getStream from "get-stream";
import engineOpts from "./options";

const chokidarOptions = { ignored: /node_modules/, ignoreInitial: true };
const separatorLine = "\n\n" + "*".repeat(80) + "\n\n";

async function execute(engine, stdin, path) {
  if (!path) {
    return engine.executeOnText(await getStream(stdin));
  }

  return engine.executeOnFiles([path]);
}

function canShortlyReportSuccess(inputFromStdin, report) {
  return (
    !inputFromStdin && report.errorCount === 0 && report.warningCount === 0
  );
}

function shortlyReportSuccess(stdout) {
  stdout.write(`\n\n👍\tat ${new Date().toLocaleString()}\n\n`);
}

function verboseReportSuccess(stdout, inputFromStdin, report) {
  const formatter = CLIEngine.getFormatter(
    inputFromStdin ? "compact" : "codeframe"
  );
  const formattedReport = formatter(report.results);
  stdout.write(`\n${formattedReport}\n`);
}

export default function({
  path,
  watch,
  stdout = process.stdout,
  stdin = process.stdin
}) {
  const inputFromStdin = !path;
  const engine = new CLIEngine(engineOpts);

  const run = async () => {
    const report = await execute(engine, stdin, path);

    if (canShortlyReportSuccess(inputFromStdin, report)) {
      shortlyReportSuccess(stdout);
    } else {
      verboseReportSuccess(stdout, inputFromStdin, report);
    }
    return report.results;
  };

  const debouncedRun = debounce(1000, true, (event, path) => {
    if ([".js", ".json"].includes(extname(path))) {
      stdout.write(separatorLine);
      run();
    }
  });

  if (watch && path) {
    chokidar.watch(path, chokidarOptions).on("all", debouncedRun);
  }

  return run();
}
